import gnome.ui
import bonobo.ui
import Bonobo
import gconf
import gnome.vfs
import sys
import os, os.path
import gettext
import locale
import gtk
import gtk.glade
import gnumexp_conf
import GNumexp_Common.gtkexcepthook
from GNumexp_Common import util
from util import *
from gettext import gettext as _

# ========================================================

APPNAME             = "GNumexp"
EXECUTABLE_NAME     = 'numexp-xygraph'
GETTEXT_PACKAGE     = 'gnumexp'
VERSION             = gnumexp_conf.VERSION
GNOMELOCALEDIR      = gnumexp_conf.GNOMELOCALEDIR
UICONFIG_DIR        = "/apps/gnumexp/UIConfig"
UICONFIG_KEY        =  UICONFIG_DIR + "/kvps"
SHOW_STATUS_BAR_KEY = UICONFIG_DIR + "/show_status_bar"
SHOW_TOOL_BAR_KEY   = UICONFIG_DIR + "/show_tool_bar"

# ========================================================

# Locale and I18N stuff
os.environ['LC_NUMERIC'] = 'C'
locale.setlocale(locale.LC_NUMERIC, 'C')
gettext.bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR)
gettext.textdomain(GETTEXT_PACKAGE)
gtk.glade.bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR)
try:
    gtk.glade.textdomain(GETTEXT_PACKAGE)
except AttributeError:
    print "gtk.glade.textdomain not found; i18n will not work"


class MainWindow(object):
    toplevels = [] # class static member

    document_uri = None # URI of the currently loaded document
    persist      = None # the Bonobo::PersistFile of the control, if any.

    def __init__(self, moniker, parent_window=None):
	self.moniker = moniker
	ui_comp = bonobo.ui.Component()
	window = bonobo.ui.Window(APPNAME, APPNAME)
	self.toplevels.append(self)
	self.window = window
	window.get_ui_engine().config_set_path(UICONFIG_KEY)
	window.connect("delete-event", self._delete_event)
	window.connect("destroy", self._window_destroyed)
	ui_container = window.get_ui_container()
	ui_comp.set_container(ui_container.corba_objref())
	bonobo.ui.util_set_ui(ui_comp, '',
			      os.path.join(gnumexp_conf.UIDIR, 'gnumexp-ui.xml'),
			      'gnumexp')
	try:
	    control = bonobo.ui.Widget(moniker, ui_container.corba_objref())
	except RuntimeError:
	    dialog = gtk.MessageDialog(
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		parent         = parent_window,
		message_format = 
		"Could not activate control from the string:\n\"%s\"" % (moniker,))
	    dialog.run()
	    dialog.destroy()
	    self.close()
	    return
	self.component = control.get_objref()
	self.persist = self.component.queryInterface("IDL:Bonobo/PersistFile:1.0")
	verbs = [
	    ("FileExit", 	self.on_menu_file_exit),
	    ("FileClose",	self.on_menu_file_close),
	    ("FileSave",	self.on_menu_file_save),
	    ("FileSaveAs",	self.on_menu_file_save_as),
	    ("FileOpen",	self.on_menu_file_open),
	    ("FileNewConsole",  self.on_menu_file_new_console),
	    ("FileNewXYGraph",	self.on_menu_file_new_xygraph),
	    ("HelpAbout", 	self.on_menu_help_about),
	    ("HelpAboutNumexp",	self.on_menu_help_about_numexp),
	    ("HelpAboutNumexp",	self.on_menu_help_about_numexp),
	    ]
	ui_comp.add_verb_list(verbs)
	ui_comp.connect('ui-event', self.__ui_event)
	window.set_contents(control)
	self._read_initial_settings(ui_comp)
	window.set_default_size(gtk.gdk.screen_width()*2/3,
				gtk.gdk.screen_height()*2/3)
	window.show_all()
	self.update_window_title()
        ui_comp.unref()

    def _window_destroyed(self, window):
	# cleanup
	if self.persist is not None:
	    self.persist.unref()
	    self.persist = None
	# remove self from global list of main windows
	self.toplevels.remove(self)
	
    def _delete_event(self, window, event):
	self.close()
	return True

    def close(self, quitting=False):
	close = self.document_save_hook(can_cancel=not quitting)
	if close:
	    self.window.destroy()

    def on_menu_file_close(self, ui_component, id):
	self.close()

    def on_menu_file_exit(self, ui_component, id):
	for mw in tuple(self.toplevels):
	    mw.close(True)

    def on_menu_file_new_console(self, ui_component, id):
	MainWindow("OAFIID:Numexp_ConsoleView")

    def on_menu_file_new_xygraph(self, ui_component, id):
	MainWindow("OAFIID:Numexp_XYGraph")

    def on_menu_help_about(self, ui_component, id):
	authors = [
	    "Gustavo Carneiro  -> main programming",
	    "Alberto Sim\303\265es  -> additional programming",
	    "Daniel Ketel  -> artwork",
	    ]
	documenters = [ "Alberto Sim\303\265es" ]
	logo = gtk.gdk.pixbuf_new_from_file(
	    os.path.join(gnumexp_conf.PIXMAPSDIR, "numexp_logo.png"))
	about = gnome.ui.About("gNumExp", gnumexp_conf.VERSION, 
			       "(C) 2001-2003 Gustavo Carneiro and Alberto Sim\303\265es",
			       "A GUI frontend to NumExp.",
			       authors, documenters, "", logo)
	about.set_transient_for(self.window)
	about.show()

    def on_menu_help_about_numexp(self, ui_component, id):
	authors = [
	    "Alberto Sim\303\265es  -> main programming",
	    "Gustavo Carneiro  -> additional programming",
	    ]
	documenters = [ "Alberto Sim\303\265es" ]
	logo = gtk.gdk.pixbuf_new_from_file(
	    os.path.join(gnumexp_conf.PIXMAPSDIR, "numexp_logo.png"))

	kernel = bonobo.get_object("OAFIID:Numexp_Kernel", "IDL:Numexp/Kernel:1.0")
	version = kernel.version()
	kernel.unref(); del kernel
	
	about = gnome.ui.About("NumExp", version, 
			       "(C) 1998-2003 Alberto Sim\303\265es and Gustavo Carneiro",
			       "A math/algebra engine and programming language.",
			       authors, documenters, "", logo)
	about.set_transient_for(self.window)
	about.show()

    def _read_initial_settings(self, ui_component):
	# status bar
	state = gconf.client_get_default().get_bool(SHOW_STATUS_BAR_KEY)
	ui_component.set_prop("/commands/ViewStatusbar", "state", util.bool_str(state))
	ui_component.set_prop("/status", "hidden", util.bool_str(not state))
	# ...
	# FileSave and FileSaveAs are only enabled if the component
	# has the Bonobo::PersistStream interface.
	if self.persist is None:
	    enable_save = "0"
	else:
	    enable_save = "1"
	ui_component.set_prop("/commands/FileSave", "sensitive", enable_save)
	ui_component.set_prop("/commands/FileSaveAs", "sensitive", enable_save)


    def __ui_event(self, ui_component, id, type, state):
 	if type == Bonobo.UIComponent.STATE_CHANGED and id == "ViewStatusbar":
 	    state = util.parse_bool(state)
	    gconf.client_get_default().set_bool(SHOW_STATUS_BAR_KEY, state)
	    ui_component.set_prop("/status", "hidden", util.bool_str(not state))

    def save_to(self, uri):
	try:
	    self.persist.save(uri)
	    save_ok = True
	except Exception, ex:
	    save_ok = False
	    bonobo_error = str(ex)#bonobo_exception_get_text(&ev);
	    dialog = gtk.MessageDialog(parent  = self.window,
				       flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
				       type    = gtk.MESSAGE_ERROR,
				       buttons = gtk.BUTTONS_OK,
				       message_format = (
		"Error saving %s:\n%s" % (uri, bonobo_error)))
	    dialog.run()
	    dialog.destroy()
	if save_ok:
	    self.document_uri = uri
	    self.update_window_title()
	return save_ok

    def load_from(self, uri):
        self.persist.load(uri)
	self.document_uri = uri
	self.update_window_title()

    def on_menu_file_save(self, ui_component, id):
	if self.document_uri is not None:
	    self.save_to(self.document_uri)
	else:
	    self.on_menu_file_save_as(None, None)

    def on_menu_file_save_as(self, ui_component, id):
	self.save_as()

    def save_as(self):
        sel = gtk.FileChooserDialog("Save...",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        sel.set_default_response(gtk.RESPONSE_OK)
        sel.set_local_only(False)

        filter = gtk.FileFilter()
        filter.set_name("Plots")
        filter.add_mime_type("application/x-xygraph")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        sel.add_filter(filter)

	sel.set_transient_for(self.window)
	while 1:
	    resp = sel.run()
	    if resp != gtk.RESPONSE_OK:
		sel.destroy()
		return False
	    uri = sel.get_uri()
            if gnome.vfs.exists(uri):
		dlg = gtk.MessageDialog(
		    flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		    parent         = sel,
		    type           = gtk.MESSAGE_WARNING,
		    buttons        = gtk.BUTTONS_YES_NO,
		    message_format = _("Overwrite '<tt>%s</tt>'?") % uri)
		dlg.vbox.get_children()[0].get_children()[1].set_property("use-markup", True)
		resp = dlg.run()
		dlg.destroy()
		if resp == gtk.RESPONSE_YES:
		    break
	    else:
		break
	sel.destroy()
	return self.save_to(uri)

    def on_menu_file_open(self, ui_component, id):
        sel = gtk.FileChooserDialog("Open...",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        sel.set_default_response(gtk.RESPONSE_OK)
        sel.set_local_only(False)

        filter = gtk.FileFilter()
        filter.set_name("gNumExp Plots")
        filter.add_mime_type("application/x-xygraph")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        sel.add_filter(filter)

	sel.set_transient_for(self.window)

	resp = sel.run()
	if resp != gtk.RESPONSE_OK:
	    sel.destroy()
	    return
	mainwindow = MainWindow("OAFIID:Numexp_XYGraph")
	mainwindow.load_from(sel.get_uri())
	sel.destroy()


    def document_save_hook(self, can_cancel):
	if self.persist is None:
	    return True
	if not self.persist.isDirty():
	    return True

	msg = "<big>%s</big>\n\n<tt>%s</tt>\n\n%s" % \
	      (_("Do you want to save the changes you made to the document?"),
	       self.document_uri,
	       _("Your changes will be lost if you don't save them."))
	msgbox = gtk.MessageDialog(parent  = self.window,
				   flags   = (gtk.DIALOG_MODAL|
					      gtk.DIALOG_DESTROY_WITH_PARENT),
				   type    = gtk.MESSAGE_QUESTION, 
				   buttons = gtk.BUTTONS_NONE, 
				   message_format = msg)
	msgbox.vbox.get_children()[0].get_children()[1].set_property("use-markup", True)

	dialog_add_button(msgbox, _("Do_n't save"), gtk.STOCK_NO, gtk.RESPONSE_NO)
	if can_cancel:
	    msgbox.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
	msgbox.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
	msgbox.set_default_response(gtk.RESPONSE_YES)

	msgbox.set_resizable(False)
	msgbox.show(); msgbox.present()
	ret = msgbox.run()
	msgbox.destroy()

	if ret == gtk.RESPONSE_YES:
	    if self.document_uri:
		save_ok = self.save_to(self.document_uri)
		close = save_ok
	    else:
		save_ok = self.save_as()
		close = save_ok
	elif ret == gtk.RESPONSE_NO:
	    close = True
	elif ret == gtk.RESPONSE_CANCEL:
	    close = False
	else:
	    close = False
	return close;

    def update_window_title(self):
	if self.moniker == 'OAFIID:Numexp_ConsoleView':
	    document = _("Console")
	elif self.moniker == 'OAFIID:Numexp_XYGraph':
	    if self.document_uri is None:
		document = _("Plot")
	    else:
		document = gnome.vfs.URI(self.document_uri).short_name
	else:
	    assert 0, "unexpected moniker " + str(self.moniker)
	self.window.set_title("%s - GNumExp" % document)
